# -*- coding: utf-8 -*-

# Copyright (c) Pilot Systems, 2013

# This file is part of Faceted SeSQL.

# SeSQL is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# SeSQL is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Faceted SeSQL.  If not, see <http://www.gnu.org/licenses/>.

from data import *

class FacetedSearch(object):
    """
    This will represent a currently running faceted search
    """
    def __init__(self, config):
        """
        Start a new faceted search for given configuration
        """
        self.config = config
        self.criteria = {}
        self.order_by = None
        self.results = {}
        self.recompute()

    def set_order(self, order_by):
        """
        Set the order to use
        """
        for order in self.config.sort_options:
            if order.name == order_by:
                self.order_by = order
                break
        else:
            self.order_by = None

    def set_criteria(self, name, value):
        """
        Set the value of a criteria
        """
        oldvalue = self.get_criteria(name)
        if oldvalue != value:
            self.criteria[name] = value
            if oldvalue:
                self.recompute()
            else:
                self.data.filter_by(name, value)

    def get_criteria(self, name):
        """
        Get the value of a criteria
        """
        return self.criteria.get(name, None)

    def recompute(self):
        """
        Recompute the values 
        """
        self.data = self.config.get_initial_data()
        self.data.search = self
        self.data.freeze_counting()
        for name, value in self.criteria.items():
            self.data.filter_by(name, value)            
        self.data.unfreeze_counting()

    def __iter__(self):
        """
        Iterate over matching sorted content
        """
        return self.data.sorted(self.order_by)

    def __len__(self):
        """
        Count matching content
        """
        return len(self.data)

    def __getitem__(self, facet):
        """
        Get a facet
        """
        return self.data[facet]

    def facets(self):
        """
        Get the list of facets
        """
        return self.data.facets.values()

    def apply_external_filter(self, fltr):
        """
        Apply an external filter, see FacetedData class for details
        """
        self.data.apply_external_filter(fltr)
