# -*- coding: utf-8 -*-

# Copyright (c) Pilot Systems, 2013

# This file is part of Faceted SeSQL.

# SeSQL is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# SeSQL is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Faceted SeSQL.  If not, see <http://www.gnu.org/licenses/>.

import copy

from django import template
from django.contrib.contenttypes.models import ContentType
from crequest.middleware import CrequestMiddleware
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from faceted_sesql.utils import pagination

register = template.Library()

@register.inclusion_tag('faceted/facets.html', takes_context = True)
def faceted_facets(context, search, *args):
    """
    Display a list of facets
    """
    current_request = CrequestMiddleware.get_request()
    return {'search': search, 'facets': args,'request': current_request}

@register.inclusion_tag('faceted/facet.html', takes_context = True)
def faceted_facet(context, search, facet):
    """  
    Display one facet (from its name)
    """
    current_request = CrequestMiddleware.get_request()
    return {'search': search, 'facet': search[facet],'current_request':current_request}

@register.simple_tag(takes_context=True)
def faceted_facet_form(context, search, facet):
    """  
    Display one facet's form
    """
    ct = ContentType.objects.get_for_model(search.config.model)
    lst = [ 'faceted/%s_%s_widget.html' % (ct.name, facet.facet.name), 
            'faceted/%s_%s_widget.html' % (ct.name, facet.facet.widget), 
            'faceted/%s_widget.html' % (facet.facet.name), 
            'faceted/%s_widget.html' % (facet.facet.widget,) ]
    t = template.loader.select_template(lst)
    context['search'] = search
    context['facet'] = facet
    return t.render(context)

@register.inclusion_tag('faceted/results.html', takes_context = True)
def faceted_results(context, search):
    """  
    Display a list of results
    """
    begin, middle, end = pagination(search.results)

    return {'search': search, 'results': search.results ,'begin' : begin, 'middle' : middle, 'end' : end}

@register.simple_tag
def faceted_item(item):
    """
    Display one item in the result
    """
    ct = ContentType.objects.get_for_model(item)
    lst = [ '%s/%s_item.html' % (ct.app_label, ct.name),
            'faceted/%s_item.html' % ct.name, 
            'faceted/item.html', ]
    t = template.loader.select_template(lst)
    context = template.Context()
    context['item'] = item
    return t.render(context)

@register.inclusion_tag('faceted/sort.html', takes_context = True)
def faceted_order(context, search):
    """
    Display the order by widget
    """ 
    selected = search.order_by
    selected_name = selected and selected.name or None
    options = []
    for opt in search.config.sort_options:
        opt = copy.copy(opt)
        opt.selected = selected_name == opt.name
        options.append(opt)
    return {'search': search, 'options': options }
